/* Ich loese die Aufgabe 1.
   Die Laufzeitangaben sind WorstCase-Laufzeiten und beziehen sich auf die Laufzeit der
   aufgerufenen Methoden.
   Der Zeitaufwand dieses Programms ist O(n+m). (Siehe Kommentar von main(String[]).)
 */
public class Aufgabe1 {
    private static Graph         graph = new Graph();
    private static List<Vertice> vlist = new List<Vertice>();


    /* Das Hauptprogramm.
       Laufzeit: O(n+m)
       Der erste Teil (bis zum Beginn der while-Schleife) ist in O(n+m) ausfuehrbar.
       (Siehe Randbemerkungen im Quellcode.)
       Die while-Schleife wird hoechstens n mal durchlaufen. Damit ist der Aufwand fuer den
       gesamten Schleifeninhalt, ausser der for-Schleife, nur O(n).
       Die for-Schleife wird insgesamt(!) nur fuer jeden besuchten Knoten fuer alle dessen
       Nachbarn einmal aufgerufen, d.h. insgesamt hoechstens 2m-mal.
       Somit ist der Aufwand fuer diesen Teil der while-Schleife nur O(m).
       Wir erhalten also einen Gesamtaufwand von O(n+m).
     */
    public static void main(String[] args) {
        graph.readfromfile(args[0], 1);                                  //Graph im Modus 1 laden: O(n+m)
        Vertice start = graph.vertice(Integer.parseInt(args[1]));       //Startknoten suchen: (1)
        Vertice ziel  = graph.vertice(Integer.parseInt(args[2]));      //Zielknoten suchen: (1)
        for (int i = 0; i < graph.verticeCount; i++) {                //Alle Knoten ...
            graph.vertices[i].visited = false;                       //... wurden noch nicht besucht ...
            graph.vertices[i].tag = new Integer(Integer.MAX_VALUE); //... und sind gaaaaaaaanz weit weg.
        }                                                          //Aufwand: O(n)

        vlist.add(start);                                        //Wir beginnen am Startknoten ...
        start.tag = new Integer(0);                             //... mit der Distanz 0.
                                                               //Aufwand: O(1)

        while (!vlist.isEmpty()) {                           //Solange es noch unbesuchte Nachbarn gibt ...
            Vertice v = vlist.sendFirst();                  //... nehmen wir uns einen Nachbarn, ...
            vlist.removeFirst();                           //... entfernen ihn aus der Warteschlange, ...
            if (!v.visited) {                             //... besuchen ihn, wenn er noch nicht besucht ...
                v.visited=true;                          //... wurde und ueberpruefen, ob ...
                if (v.id == ziel.id) {                  //... wir am Ziel sind.
                    System.out.println(((Integer) v.tag).intValue()); //Am Ziel => Ausgabe des Abstands.
                    System.exit(0);                                  //Fertig.
                }
                for (int i = 0; i < v.neighbours.length; i++) {    //Nicht am Ziel => Fuer alle ...
                    Vertice w=v.neighbours[i].otherEnd(v);        //... Nachbarn von v, die ...
                    if (!w.visited) {                            //... noch nicht besucht sind, ...
                        int d_v = ((Integer) v.tag).intValue(); //... berechnen wir den neuen Abstand ...
                        int d_w = ((Integer) w.tag).intValue();
                        w.tag = new Integer(Math.min(d_v + 1, d_w)); 
                        vlist.add(w);         //... und fuegen sie der Warteschlange hinzu.
                    }
                }
            }
        }
        System.out.println(-1);          //Start und Ziel sind nicht in der
    }                                   //selben Zusammenhangskomponente.
}