import java.util.Comparator;





/* Ich loese die Aufgabe 3.
   Die Laufzeitangaben sind WorstCase-Laufzeiten und beziehen sich auf die Laufzeit der
   aufgerufenen Methoden.
   n ist immer die Anzahl der Knoten, die auch mindestens eine Kante haben. Alle anderen
   Knoten werden weder geladen, noch initialiert und auch sonst nie in irgendeiner Weise
   beruehrt. Damit gilt n<=2*m (m ist die Anzahl der Kanten im Graphen) und somit auch
   n=O(m).
   Der Aufwand dieses Programms ist O(n*m*log(n)). (Siehe Kommentar von main(String[]).)
 */
public class Aufgabe3 {
    private static Graph graph = new Graph();            //Hier kommt der Graph rein
    private static int   minLength = Integer.MAX_VALUE; //die Laenge des kuerzesten bisher
                                                       //gefundenen Kreises



    /* Diese Methode laedt den Graphen, ueberprueft, ober er ein Wald ist und gibt ggf.
       die Laenge eines kleinsten Kreises aus.
       Laufzeit:
        -Die Initialisierung erfolgt in O(m*log(m)+n)=O(m*log(m)).
        -Der Wald-Test hat einen Aufwand von O(n+m).
        -Die Suche nach einem kleinsten Kreis wird fuer jeden Knoten einmal aufgerufen.
         Der Aufwand ist also O(n*(Aufwand(minCirc)+log(n)+log(m)^2)=O(n*m*log(n)).
        Der Gesamtaufwand fuer den Algorithmus ist somit O(n*m*log(n)).
     */
    public static void main(String[] args) {
        graph.readfromfile(args[0], 0);      //Graph im Modus 0 laden: O(m*log(m))
        graph.createCreatedVerticesArray(); //O(n), wobei n <= 2*m, da wir nur die Knoten
                                           //anfassen, die Kanten haben. :-)

        if (wald()) {                    //Wenn der Graph ein Wald ist, dann sind wir fertig.
            System.out.println(-1);     //Aufwand fuer diese Abfrage: O(n+m)
            System.exit(0);
        }

        //Suche nach dem kleinsten Kreis
        for (int i = 0; i < graph.createdV.length; i++) {                 //Suche fuer jeden Knoten
            minLength = Math.min(minLength, minCirc(graph.createdV[i])); //den kleinsten Kreis, in
                                                                        //dem er enthalten ist. Der
                                                                       //Aufwand fuer die Kreissuche
                                                                      //ist O(m*log(n)).
            graph.removeVertice(graph.createdV[i].id);               //Knoten, die nicht im kleinsten
        }                                                           //Kreis enthalten sind, brauchen
                                                                   //wie sowieso nicht mehr. Der
                                                                  //Aufwand fuer das Entfernen des
                                                                 //Knotens ist O(log(n)+log(m)^2)

        System.out.println(minLength);                         //Ergebnis ausgeben.
    }



    /* gibt genau dann true zurueck, wenn der Graph ein Wald ist.
       Dabei muessen nur die Knoten ueberprueft werden, die auch Kanten haben. Die anderen
       Knoten sind ganz sicher nicht Teil eines Kreises.
       Laufzeit:
        Die for-Schleifen werden fuer jeden Knoten genau einmal aufgerufen. => O(n)
        Die Methode besucheWvonV(Vertice, int) nur fuer unbesuchte Knoten aufgerufen. (Dies
        gilt auch fuer die rekursiven Aufrufe in der Methode selbst.) Damit finden O(n)
        Aufrufe dieser Methode statt.
        Bei der Ausfuehrung aller Aufrufe von besucheWvonV(Vertice, int) werden alle Kanten
        insgesamt hoechstens zweimal besucht, womit wir einen zusaetzlichen Aufwand von O(m)
        erhalten.
        Somit hat die Methode wald() einen Aufwand von O(n+m).
     */
    private static boolean wald() {
        for (int i = 0; i < graph.createdV.length; i++) {
            graph.createdV[i].visited = false;               //Noch haben wir keinen Knoten besucht.
            graph.createdV[i].createArray();                //Erzeugt das Adjazenz-Array des Knotens,
        }                                                  //damit wir dessen Nachbarn schneller finden.
        for (int i = 0; i < graph.createdV.length; i++)   //Alle Knoten nacheinander besuchen, die
            if (!graph.createdV[i].visited)              //noch nicht besucht wurden.
                if (besucheWvonV(graph.createdV[i], 0)) //Wenn ein Kreis in dieser Zusammenhangs-
                    return false;                      //komponente gefunden wird, ist der Graph
                                                      //kein Wald.
        return true;
    }



    /* besucht vom Knoten v aus den Nachbarn w. Dabei wird dafuer gesorgt, dass beim Besuchen
       der Nachbarn von w nicht sofort wieder der Knoten v besucht wird, indem dessen id mit
       uebergeben wird. Es werden also nur alle Nachbarn von w besucht, die nicht v sind (bzw.
       die nicht die id-Nummer von v haben). Wenn alle Nachbarn besucht werden sollen, kann
       man einfach den Wert 0 als id von v uebergeben, da die Knoten-ids in dem Graphen alle
       groesser als 0 sind. Somit werden bei id_v==0 alle Nachbarn von w besucht, was genau
       dann noetig ist, wenn w der erste Knoten einer Zusammenhangskomponente ist.
       Der Rueckgabewert ist genau dann true, wenn beim Aufbauen des Tiefensuchbaumes von w
       ausgehend ein Kreis gefunden wird.
       Laufzeit: Siehe wald().
     */
    private static boolean besucheWvonV(Vertice w, int id_v) {
        w.visited = true;                                      //w wird besucht.
           for (int i = 0; i < w.neighbours.length; i++)      //Alle Nachbarn von w, die nicht die id
               if (w.neighbours[i].otherEnd(w).id != id_v) { //von v haben, ebenfalls besuchen.
                   if (w.neighbours[i].otherEnd(w).visited) //Wenn der Nachbar schon besucht wurde,
                       return true;                        //hat der Graph einen Kreis.
                   if (besucheWvonV(w.neighbours[i].otherEnd(w), w.id)) //Alle Nachbarn besuchen und
                       return true;                                    //true zurueckgeben, wenn ein
               }                                                      //Kreis gefunden wird.
        return false;                                                //Es wurde kein Kreis gefunden.
    }



    /* sucht nach einem kleinsten Kreis, der den Knoten v enhaelt und entfernt anschliessend
       alle Kanten des Knotens v. (Das beschleunigt die Suche bei zukuenftigen Knoten etwas,
       da dann weniger Kanten betrachtet werden muessen.)
       Das Entfernen der Kanten von v beeinflusst das Ergebnis der Suche nicht, weil:
        1. Fall: v ist in einem kleinsten Kreis des Graphen enthalten.
         Dann ist die Laenge eines kleinsten Kreises des Graphen bereits bei der Suche nach
         einem kleinsten Kreis, der v enthaelt, bestimmt worden.
        2. Fall: v ist nicht in einem kleinsten Kreis des Graphen enthalten.
         Dann ist auch keine Kante von v in einem kleinsten Kreis des Graphen enthalten, also
         muessen wir diese in Zukunft auch nicht mehr weiter betrachten.
       Wie wird nun die Laenge dieses Kreises bestimmt?
       Wir starten einfach vom Knoten v ausgehend eine abgewandelte Form von Dijkstra:
        Wie im bekannten Dijkstra-Algorithmus wird der Abstand jedes Knotens von v bestimmt.
        Wenn wir nun einen Knoten x besuchen und dabei bei einem von seinen Nachbarn (nennen
        wir diesen y) das Gewicht nach unten korrigieren, haben wir einen Kreis geschlossen.
        Dieser Kreis ist hoechstens so lang, wie die Summe der Laengen der Pfade von v zu x
        und y und dem Gewicht der Kante zwischen x und y.
        (Es ist dabei durchaus moeglich, dass der Knoten v nicht auf dem Kreis liegt und der
        entdeckte Kreis damit in Wirklichkeit sogar noch kleiner ist, aber sobald wir spaeter
        einen der Knoten des Kreises als Startpunkt von Dijkstra waehlen, werden wir die
        genaue Kreislaenge bestimmen.)
        Es gibt also zwei Faelle:
         1. Fall: v liegt auf einem kleinsten Kreis.
          Dann koennen wir keine kuerzeren Kreise finden. Insbesondere ist die vorherige
          Bemerkung in den Klammern in diesem Fall ueberfluessig.
         2. Fall: v liegt nicht auf einem kleinsten Kreis.
          In diesem Fall erhalten wir als Kreislaenge die Laenge eines gefundenen Kreises
          plus das Doppelte der Enfernung dieses Kreises von v. Auf jeden Fall werden wir
          jedoch keinen Wert als Kreislaenge erhalten, der kleiner als ein kuerzester Kreis
          im Graphen ist.
        Das Minum der Kreislaengen, die wir auf diese Weise bestimmen wird immer in curlength
        gespeichert. Sobald wir nur noch Knoten betrachten, die weiter als curlength/2 von v
        entfernt sind, muessen wir die Suche nicht mehr fortsetzen, da wir nur noch Kreise
        finden koennen, die mindestens so lang wie curlength sind. (Daher beschleunigt die
        zweite Abbruchbedingung der while-Schleife die Suche etwas, ohne das Ergebnis zu
        verfaelschen. Leider ist dies kein Geschwindigkeitsvorteil, der in der O-Notation
        auffaellt.)
       Laufzeit:
        -Die Initialisierung der Knoten erfolgt in O(n).
        -Der Aufbau des AVL-Baumes, wo wir die noch zu besuchenden Knoten speichern, erfolgt
         in O(n*log(n)).
        -Die while-Schleife wird fuer jeden Knoten genau einmal ausgefuehrt.
         Damit ist der Aufwand von allen Operationen, die sich nicht in weiteren inneren
         Schleifen befinden, O(n).
        -Die innere for-Schleife wird fuer jeden Knoten fuer jeden seiner Nachbarn, also
         insgesamt O(m)-mal ausgefuehrt.
         Die darin befindlichen AVL-Baum-Operationen sind in O(log(n)) ausfuehrbar.
         Somit haben wir einen Gesamtaufwand von O(m*log(n)) fuer die for-Schleife.
        Der Gesamtaufwand ist also O(n+n*log(n)+n+m+m*log(n))=O((n+m)*log(n))=O(m*log(n)),
        da n=O(m).
     */
    private static int minCirc(Vertice v) {
        //Speichert die Knoten in der Reihenfolge ihrer Entfernung vom Ausgangsknoten.
        AVLTree<Vertice> zubesuchen = new AVLTree<Vertice>(new NodeDistComparator());
        //Speichert das Minimum der maximalen Laengen der Kreise, die wir geschlossen haben.
        int              curlength  = Integer.MAX_VALUE;

         //Von v ausgehend starten wir Dijkstra und brechen ab, sobald wir keinen Kreis mehr
        //schliessen koennen, der kurzer als curlength ist und v enthaelt.
        for (int i = 0; i < graph.createdV.length; i++) {                //dist=\infty fuer
            graph.createdV[i].tag     = new Integer(Integer.MAX_VALUE); //alle Knoten des Graphen
            graph.createdV[i].visited = false;                         //ausserdem noch nicht besucht
        }
        v.tag = new Integer(0);                                      //dist(v)=0
        for (int i = 0; i < graph.createdV.length; i++)             //Wir muessen noch alle Knoten
            zubesuchen.insert(graph.createdV[i]);                  //besuchen.

        while (   (!zubesuchen.isEmpty())                        //So lange es noch wartende Knoten
               && (Integer)zubesuchen.min().tag * 2 <= curlength) {
            Vertice x = zubesuchen.extractmin();               //gibt, nehmen wir einen Knoten x mit
            x.visited = true;                                 //der geringsten Entfernung zu v heraus.
            for (int j = 0; j < x.neighbours.length; j++) {  //Fuer alle Nachbarn von x, die noch
                Vertice y = x.neighbours[j].otherEnd(x);    //nicht verarbeitet wurden, aktualisieren
                if (!y.visited) {                          //wir deren Entfernung:
     
                    curlength = Math.min(
                                  curlength,
                                  boundedAdd(
                                    ((Integer)y.tag).intValue(),     //Kreise, die v enthalten
                                    boundedAdd(
                                      ((Integer)x.tag).intValue(), //haben hoechstens diese Laenge.
                                      x.neighbours[j].weight)));

                    zubesuchen.remove(y);                       //Knoten aus dem Baum holen
                    y.tag     = new Integer(Math.min(          //Abstand aktualisieren
                                              ((Integer)y.tag).intValue(),
                                              boundedAdd(
                                                ((Integer)x.tag).intValue(),
                                                x.neighbours[j].weight)));
                    zubesuchen.insert(y);                 //Knoten wieder einsortieren
                }
            }
        }
        return curlength;
    }



    /* addiert zwei nichtnegative Zahlen. Wenn die Summe einen Ueberlauf verursacht, wird
       Integer.MAX_VALUE zurueckgegeben.
       Alle Methoden haben einen Aufwand von O(1).
     */
    private static int boundedAdd(int s1, int s2) {
        return (s1 + s2 < 0? Integer.MAX_VALUE : s1 + s2);
    }




    /* Ich bin eine Vergleichs-Black-Box fuer Knoten, welche diese erst nach ihrem Abstand
       (als Integer im tag-Attribut gespeichert) und anschliessend nach ihrer ID vergleicht.
     */
    private static class NodeDistComparator implements Comparator<Vertice> {


        /* Die Vergleichsmethode.
           Laufzeit: O(1)
         */
        public int compare(Vertice v1, Vertice v2) {
            int i = ((Integer) v1.tag).intValue() - ((Integer) v2.tag).intValue();
            return (i == 0? v1.id - v2.id: i);
        }
    }
}