/* Ich bin eine Liste.
   Diese (vor- und rueckwaertsverkettete) Liste enthaelt unsortierte Ansammlungen von
   Elementen.
   Alle Laufzeitangaben sind WorstCase-Angaben und n ist die Anzahl der Elemente.
 */
public class List<E> {
    private ListElement first     = null;  //das erste Element
    private ListElement last      = null; //das letzte Element
    private int         listCount = 0;   //Anzahl der Elemente



    /* erzeugt eine neue Liste.
       Laufzeit: O(1)
     */
    public List() {}



    /* haengt ein neues Element an die Liste an.
       Laufzeit: O(1)
     */
    public void add(E wert){
        if (listCount == 0) {                  //1. Fall: Liste ist leer
            first       = new ListElement();
            first.value = wert;
            last        = first;
        } else {                           //2. Fall: Liste ist nicht leer
            ListElement temp = new ListElement();
            temp.previous = last;
            temp.value    = wert;
            last.next     = temp;
            last          = temp;
        }
        listCount++;
    }



    /* gibt die Anzahl der Elemente der Liste zurueck.
       Laufzeit: O(1)
     */
    public int count() {
        return listCount;
    }



    /* gibt genau dann true zurueck, wenn die Liste leer ist.
       Laufzeit: O(1)
     */
    public boolean isEmpty() {
        return listCount == 0;
    }


    /* gibt das erste Element der Liste zurueck.
       Laufzeit: O(1)
     */
    public E sendFirst() {
        return first.value;
    }



    /* rotiert die Liste, d.h. das erste Element wird hinten angehaengt.
       Diese Methode kann z.B. verwendet werden, um die gesamte Liste durchsuchen zu koennen.
       Laufzeit: O(1)
     */
    public void rotate() {
        if (listCount == 0)
            return;
        last.next      = first;
        first.previous = last;
        last           = first;
        first          = first.next;
    }



    /* gibt das letzte Element der Liste zurueck.
       Laufzeit: O(1)
     */
    public E sendLast() {
        return last.value;
    }



    /* entfernt das erste Element der Liste.
       Laufzeit: O(1)
     */
    public void removeFirst() {
        first = first.next;
        listCount--;
        if (listCount == 0)           //Wenn die Liste leer ist, ...
            last = null;             //... dann gibt's kein letztes Element mehr, ...
        else                        //... sonst ...
            first.previous = null; //... sagen wir dem neuen ersten Element, dass es das erste Element ist.
    }



    /* entfernt das letzte Element der Liste.
       Laufzeit: O(1)
     */
    public void removeLast() {
        last = last.previous;
        listCount--;
        if (listCount == 0)      //Wenn die Liste leer ist, ...
            first = null;       //... dann gibt's kein erstes Element mehr, ...
        else                   //... sonst ...
            last.next = null; //... sagen wir dem neuen letzten Element, dass es das letzte Element ist.
    }



    /* fuellt das Array ziel mit dem Inhalt der Liste.
       Das Array muss gross genug sein, um den gesamten Listeninhalt aufzunehmen.
       Laufzeit: O(n)
     */
    public void toArray(E[] ziel) {
        ListElement aktuell = first;
        for (int i = 0; i < listCount; i++){    //Fuer jedes Listenelement ...
            ziel[i] = aktuell.value;           //... tragen wir das Element in das Array ein ...
            aktuell = aktuell.next;           //... und machem mit den naechsten Element weiter.
        }
    }



    /* wandelt die Liste in eine String-Darstellung um.
       Die Elemente werden mit <E>.toString() dargestellt.
       Laufzeit: O(n)
     */
    public String toString() {
        String      wert    = "[";
        ListElement aktuell = first;
        for (int i = 0; i < listCount; i++) {               //Fuer jedes Listenelement ...
            wert   += (i == 0? "" : ", " + aktuell.value); //... schreiben wir das Element in den String ...
            aktuell = aktuell.next;                       //... und machem mit den naechsten Element weiter.
        }
        return wert + "]";
    }



    /* erstellt eine Liste mit dem Inhalt des Arrays.
       der bisherige Listeninhalt wird verworfen.
       Laufzeit: O(n)
     */
    public void fromArray(E[] quelle) {
        first     = null;
        last      = null;
        listCount = 0;
        for (int i = 0; i < quelle.length; i++) //Fuege einfach jedes Element des Arrays' ein.
            add(quelle[i]);
    }





    /* Ich bin ein Listenelement.
       Es gibt (ausser dem O(1)-Konstruktor, etc. ) keine Methoden, die irgendeine Laufzeit
       haben koennten.
     */
    private class ListElement {
        public ListElement next     = null;  //Nachfolger
        public ListElement previous = null; //Vorgaenger
        public E           value;          //Inhalt
    }
}